
#include <EEPROM.h>
#include <TimerOne.h>
#include <TM1637Display.h>

/*-----------------------------------------------------------------------------------------------------*/
#define CLK   12
#define DIO   11 

TM1637Display display(CLK, DIO);      // Set up the 4-Digit Display.

uint8_t data[]  = {0, 0, 0, 0};

int  Digit_0[]  = {0xBF,0x86,0xDB,0xCF,0xE6,0xED,0xAD,0x87,0xFF,0xEF};
int  Digit_1[]  = {0x00,0x00,0xCF,0xCF,0xF3,0xB9,0xEE,0xEE,0xBF,0x00};
int  Digit_2[]  = {0x00,0x06,0x5B,0x4F,0x66,0x6D,0x7D,0x07,0x7F,0x6F};
int  Digit_3[]  = {0x3F,0x06,0x5B,0x4F,0x66,0x6D,0x7D,0x07,0x7F,0x6F};
/*-----------------------------------------------------------------------------------------------------*/
//#define Fuse            2                                 // Вход прерывания. Датчик Фазы
#define AC_LOAD           6                                 // Вывод для управления симистором
//#define OUT             5                                 // Пал смертью храбрых от 12 вольт

#define KEY_Plus          A3
#define KEY_Select        A5
#define KEY_Minus         A4
#define KEY_Start         A2
#define Led               4

int  PWM[]                = {0, 0};
byte Fuse_Pause_ON        = 4;

unsigned long time;
unsigned long timenext    = 0;

byte Start_ON             = 0;
byte digitBuffer[]        = {5, 1, 0, 5, 0};

byte Impuls               = 1;
byte Select               = 1;
byte Second_Impuls        = 0;
int  Sec[]                = {0, 0, 0};
byte Write                = 0;
/*=======================================================================================================
                                              Чтение EEPROM
=======================================================================================================*/
void EEPROM_Read()
{
  for (int i = 0; i <= 4; i++)
  {
    digitBuffer[i] = EEPROM.read(i);                // считываем значение по текущему адресу EEPROM
  }
  Fuse_Pause_ON = digitBuffer[0];
  PWM[0] = (digitBuffer[1] * 1000) + (digitBuffer[2] * 100) + (digitBuffer[3] * 10) + digitBuffer[4];
  PWM[1] = (digitBuffer[2] * 100) + (digitBuffer[3] * 10) + digitBuffer[4];
  Impuls = digitBuffer[1];
}
/*=======================================================================================================
                                              Запись EEPROM
=======================================================================================================*/
void EEPROM_Write()
{
  for (int i = 0; i <= 4; i++)
  {
    EEPROM.write(i, digitBuffer[i]);
  }
}
/*=======================================================================================================
                                    Функция включения симистора по прерыванию                               
=======================================================================================================*/
void Front_Up()                                       // Прерывание по изменению уровня с низкого на высокий
{
  delay(Fuse_Pause_ON);                               // Пауза для включения симистора на пике волны
  digitalWrite(AC_LOAD, HIGH);                        // Включить симистор
  detachInterrupt(0);     
}
/*=======================================================================================================
                                              Инициализация
=======================================================================================================*/
void setup()
{
  Serial.begin(115200);

  display.setBrightness(0x0F);
  display.showNumberDec(1234);

  pinMode(AC_LOAD, OUTPUT);
  digitalWrite(AC_LOAD, LOW);
  
  pinMode(KEY_Plus,  INPUT_PULLUP);
  pinMode(KEY_Select, INPUT_PULLUP);
  pinMode(KEY_Minus, INPUT_PULLUP);
  pinMode(KEY_Start, INPUT_PULLUP);
  
  pinMode(Led,  OUTPUT); 
  digitalWrite(Led, LOW);
  delay(300);
  digitalWrite(Led, HIGH);
  
//  EEPROM_Write();
  EEPROM_Read();
  Timer1.initialize(50000);                           // Timing for 50 ms
  Timer1.attachInterrupt(TimingISR);                  // Declare the interrupt serve routine : TimingISR  
}
/*=======================================================================================================
                                              Главный цикл
=======================================================================================================*/
void loop()
{
  ShowDisplay();

  time = millis();  
  if (time >= timenext)
  {
    timenext += PWM[1];                               // Длительность импульса
    digitalWrite(AC_LOAD, LOW);                       // Выключение симистора(он выключится когда полупериод достигнет нуля)
    digitalWrite(Led, HIGH);
    if (Second_Impuls == 0) {Sec[0] = 0; Second_Impuls = 1;}
  }
}
/*=======================================================================================================
                                              Динамическая индикация
=======================================================================================================*/
void ShowDisplay()
{
  for (byte i = 0; i <= 9; i++)
  {
    if (digitBuffer[1] == i) data[0] = Digit_3[i];
    if (digitBuffer[2] == i) data[1] = Digit_1[i];
    if (digitBuffer[3] == i) data[2] = Digit_2[i];
    if (digitBuffer[4] == i) data[3] = Digit_3[i];
  }
}
/*=====================================================================================================*/
